<?php
namespace Controller;

use Exception\CustomException;
use Util\Instance\Service;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Messages;
use Zeedhi\Framework\DTO;
use Zeedhi\DevOps\Util\Utilities;

class CallComment extends Simple
{
    /** @var  \Service\CallComment */
    protected $callCommentService;

    function __construct(\Service\CallComment $callCommentService) {
        $this->callCommentService = $callCommentService;
    }

    public function saveSupportCallComment(Request\DataSet $request, Response $response) {
        try {
            $supportCallComment = $request->getDataSet()->getRows();
            $supportCallComment = $supportCallComment[0];
            if(!empty($supportCallComment)) {
                $description        = $supportCallComment["DESCRIPTION"];
                $createdBy          = $supportCallComment["CREATED_BY"];
                $callRelationship   = $supportCallComment["CALL_RELATIONSHIP"];
                $this->callCommentService->saveSupportCallComment($description, $createdBy, $callRelationship);
            } else {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_COMMENT', 500, array( 'error' => ''));
            }
        } catch (CustomException $e) {
            $response->setError(\Util\Utilities::getErrorObjectFromException($e));
        }
    }

    public function supportCallComments(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        if(!empty($params)) {
            $callId             = Utilities::getValueInFilterDataSourceByName($params,'CALL_ID');
            $callRelationship   = Utilities::getValueInFilterDataSourceByName($params,'CALL_RELATIONSHIP');
            $supportCallComments = $this->callCommentService->findSupportCallComments($callId, $callRelationship);
            $response->addDataSet(new DataSet('supportCallComments', $supportCallComments));
        } else {
            $response->addDataSet(new DataSet('supportCallComments', array()));
        }
    }

    public function saveProductCallComment(Request\DataSet $request, Response $response) {
        try {
            $callComment = $request->getDataSet()->getRows();
            $callComment = $callComment[0];
            if(!empty($callComment)) {
                $subject          = isset($callComment["SUBJECT"]) ? $callComment["SUBJECT"] : '';
                $description      = $callComment["DESCRIPTION"];
                $callId           = $callComment["CALL_ID"];
                $callRelationship = isset($callComment["CALL_RELATIONSHIP"]) ? $callComment["CALL_RELATIONSHIP"] : null;
                $callType         = isset($callComment["CALL_TYPE"]) ? $callComment["CALL_TYPE"] : null;
                $this->callCommentService->saveProductCallComment($description, null, $callId, $subject, $callRelationship, $callType);
            } else {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_COMMENT', 500, array( 'error' => ''));
            }
        } catch (CustomException $e) {
            $response->setError(\Util\Utilities::getErrorObjectFromException($e));
        }
    }

    public function responseQuestionIssue(Request\DataSet $request, Response $response) {
        try {
            $callComment = $request->getDataSet()->getRows();
            $callComment = $callComment[0];
            if(!empty($callComment)) {
                $subject          = isset($callComment["SUBJECT"]) ? $callComment["SUBJECT"] : '';
                $description      = $callComment["DESCRIPTION"];
                $callId           = $callComment["CALL_ID"];
                $callRelationship = isset($callComment["CALL_RELATIONSHIP"]) ? $callComment["CALL_RELATIONSHIP"] : null;
                $callType         = isset($callComment["CALL_TYPE"]) ? $callComment["CALL_TYPE"] : null;
                $documents         = isset($callComment["DOCUMENT"]) ? $callComment["DOCUMENT"] : array();
                Service::getAttachment()->responseQuestionIssue($callId, $description, $documents,  $callRelationship, $subject, $callType);
            } else {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_COMMENT', 500, array( 'error' => ''));
            }
        } catch (CustomException $e) {
            $response->setError(\Util\Utilities::getErrorObjectFromException($e));
        }
    }
    
    public function findCallComments(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        if (!empty($params)) {
            $callRelationship = Utilities::getValueInFilterDataSourceByName($params,'CALL_RELATIONSHIP');
            $callType         = Utilities::getValueInFilterDataSourceByName($params,'CALL_TYPE');
            $callComments     = $this->callCommentService->findCallComments( $callRelationship, $callType);
            $response->addDataSet(new DataSet('productCallComments', $callComments));
        } else {
            $response->addDataSet(new DataSet('productCallComments', array()));
        }
    }

    public function productCallComments(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        if (!empty($params)) {
            $callRelationship = Utilities::getValueInFilterDataSourceByName($params,'CALL_RELATIONSHIP');
            $callType         = Utilities::getValueInFilterDataSourceByName($params,'CALL_TYPE');
            $productCallComments = $this->callCommentService->findProductCallComments($callRelationship, $callType);
            $response->addDataSet(new DataSet('productCallComments', $productCallComments));
        } else {
            $response->addDataSet(new DataSet('productCallComments', array()));
        }
    }
} 