/************************************************************************************
****    Procedure para registro do perodo para apurao do resultado operacional ***
*************************************************************************************

Criado em  : 08/2007
       Por : Alessandra Cristina

Descrio:

           - Altera perodo programado para apurao do Resultado Operacional da Filial
             de acordo com a nova data de processamento parametrizada.

           - Caso a data j tenha sido apurada deleta registro da tabela ROPFILIAL, tabela
             utilizada para restringir reclculo do ROP.
*/

IF EXISTS(SELECT
             NAME
          FROM
             SYSOBJECTS
          WHERE
             NAME = 'ATUALIZA_DADOS_DTPROCESSA' AND
             TYPE = 'TR')


  DROP TRIGGER ATUALIZA_DADOS_DTPROCESSA
  GO


CREATE TRIGGER ATUALIZA_DADOS_DTPROCESSA
ON PARAMFILIAL
FOR UPDATE
AS

   DECLARE @V_DTINICIAL       DATETIME;
   DECLARE @V_DTFINAL         DATETIME;
   DECLARE @V_IDFECHDIAROP    VARCHAR(1);
   DECLARE @V_NEW_CDFILIAL    VARCHAR(4);
   DECLARE @V_OLD_CDFILIAL    VARCHAR(4);
   DECLARE @V_NEW_DTPROCESSA  DATETIME;
   DECLARE @V_OLD_DTPROCESSA  DATETIME;
   DECLARE @V_ID_COMANDO      VARCHAR(1);
   DECLARE @V_IDUTPADRROP     VARCHAR(1);
   DECLARE @V_IDUTPADRROPFIL  VARCHAR(1);

BEGIN
   SET @V_ID_COMANDO = 'N';

   DECLARE C_CURSOR_DATA CURSOR FOR
      SELECT
        'U' IDCOMANDO,
        DELETED.CDFILIAL,
        DELETED.DTPROCESSA,
        INSERTED.CDFILIAL,
        INSERTED.DTPROCESSA
      FROM
          DELETED,
          INSERTED
      WHERE
         INSERTED.CDFILIAL  = DELETED.CDFILIAL;
   OPEN C_CURSOR_DATA;
   FETCH NEXT FROM C_CURSOR_DATA INTO @V_ID_COMANDO,
                                      @V_OLD_CDFILIAL,
                                      @V_OLD_DTPROCESSA,
                                      @V_NEW_CDFILIAL,
                                      @V_NEW_DTPROCESSA;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      IF @V_ID_COMANDO = 'U' BEGIN
         SELECT @V_IDUTPADRROP = IDUTPADRROP
         FROM PARAMGERAL G;

         SELECT @V_IDFECHDIAROP   = ISNULL(MAX(IDFECHDIAROP),'N'),
                @V_IDUTPADRROPFIL = ISNULL(MAX(IDUTPADRROPFIL),'N')
         FROM  PARFILIALROP
         WHERE CDFILIAL = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL);

         IF @V_IDUTPADRROP = 'S' OR @V_IDUTPADRROPFIL = 'S' BEGIN
            SELECT @V_IDFECHDIAROP   = ISNULL(MAX(IDFECHDIAROP),'N'),
                   @V_IDUTPADRROPFIL = ISNULL(MAX(IDUTPADRROPFIL),'N')
            FROM  PARFILIALROP
            WHERE CDFILIAL = 'P';
         END;

         IF (@V_NEW_DTPROCESSA <> @V_OLD_DTPROCESSA) BEGIN
            SELECT @V_DTINICIAL = MAX(DTINIAPURACROP),
                   @V_DTFINAL   = MAX(DTFINAPURACROP)
            FROM APURACAOROP
            WHERE CDFILIAL = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL);

            IF (ISNULL(@V_IDFECHDIAROP,'N') <> 'S') BEGIN
               IF (@V_NEW_DTPROCESSA > @V_DTFINAL) BEGIN
                  UPDATE APURACAOROP
                  SET DTFINAPURACROP  = @V_NEW_DTPROCESSA
                  WHERE CDFILIAL = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL);

                  DELETE ROPFILIAL
                  WHERE CDFILIAL     = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL)  AND
                        DTFILIALROP  >= @V_NEW_DTPROCESSA;
               END;

               IF (@V_NEW_DTPROCESSA < @V_DTFINAL)  AND (@V_NEW_DTPROCESSA < @V_DTINICIAL) BEGIN

                  UPDATE APURACAOROP
                  SET DTINIAPURACROP  = @V_NEW_DTPROCESSA
                  WHERE CDFILIAL = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL);
               END;
            END
            ELSE BEGIN
               DELETE ROPFILIAL
               WHERE CDFILIAL     = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL)  AND
                     DTFILIALROP  >= @V_NEW_DTPROCESSA;

               IF (@V_NEW_DTPROCESSA < @V_DTFINAL)  BEGIN
                  UPDATE APURACAOROP
                  SET  DTFINAPURACROP  = @V_NEW_DTPROCESSA
                  WHERE CDFILIAL = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL);
               END;
               IF (@V_NEW_DTPROCESSA < @V_DTINICIAL) BEGIN
                  UPDATE APURACAOROP
                  SET  DTINIAPURACROP  = @V_NEW_DTPROCESSA
                  WHERE CDFILIAL = ISNULL(@V_NEW_CDFILIAL, @V_OLD_CDFILIAL);
               END;

            END;
         END;
      END;
      FETCH NEXT FROM C_CURSOR_DATA INTO @V_ID_COMANDO,
                                         @V_OLD_CDFILIAL,
                                         @V_OLD_DTPROCESSA,
                                         @V_NEW_CDFILIAL,
                                         @V_NEW_DTPROCESSA;

   END;
   CLOSE C_CURSOR_DATA;
   DEALLOCATE C_CURSOR_DATA;
END;

GO

-- VERSAO --
EXEC ATUVEROBJ 'ATUALIZA_DADOS_DTPROCESSA','5.01.011';
GO

